@@ -12,7 +12,6 @@ extern "C" {
 #include <sdb.h>
 
 #include <stdio.h>
-#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -26,6 +25,7 @@ extern "C" {
 #include <windows.h>
 #include <wincon.h>
 #endif
+#include <unistd.h>
 
 /* constants */
 #define CONS_MAX_USER 102400
@@ -58,6 +58,7 @@ typedef struct r_cons_grep_t {
 	bool charCounter;
 	int less;
 	int json;
+	char *json_path;
 	int range_line;
 	int line;
 	int sort;
@@ -71,9 +72,155 @@ typedef struct r_cons_grep_t {
 	int neg;
 	int begin;
 	int end;
+	int icase;
 } RConsGrep;
 
+#if 0
+// TODO Might be better than using r_cons_pal_get_i
+// And have smaller RConsPrintablePalette and RConsPalette
+enum {
+	R_CONS_PAL_0x00 = 0,
+	R_CONS_PAL_0x7f,
+	R_CONS_PAL_0xff,
+	R_CONS_PAL_ARGS,
+	R_CONS_PAL_BIN,
+	R_CONS_PAL_BTEXT,
+	R_CONS_PAL_CALL,
+	R_CONS_PAL_CJMP,
+	R_CONS_PAL_CMP,
+	R_CONS_PAL_COMMENT,
+	R_CONS_PAL_CREG,
+	R_CONS_PAL_FLAG,
+	R_CONS_PAL_FLINE,
+	R_CONS_PAL_FLOC,
+	R_CONS_PAL_FLOW,
+	R_CONS_PAL_FLOW2,
+	R_CONS_PAL_FNAME,
+	R_CONS_PAL_HELP,
+	R_CONS_PAL_INPUT,
+	R_CONS_PAL_INVALID,
+	R_CONS_PAL_JMP,
+	R_CONS_PAL_LABEL,
+	R_CONS_PAL_MATH,
+	R_CONS_PAL_MOV,
+	R_CONS_PAL_NOP,
+	R_CONS_PAL_NUM,
+	R_CONS_PAL_OFFSET,
+	R_CONS_PAL_OTHER,
+	R_CONS_PAL_POP,
+	R_CONS_PAL_PROMPT,
+	R_CONS_PAL_PUSH,
+	R_CONS_PAL_CRYPTO,
+	R_CONS_PAL_REG,
+	R_CONS_PAL_RESET,
+	R_CONS_PAL_RET,
+	R_CONS_PAL_SWI,
+	R_CONS_PAL_TRAP,
+	R_CONS_PAL_AI_READ,
+	R_CONS_PAL_AI_WRITE,
+	R_CONS_PAL_AI_EXEC,
+	R_CONS_PAL_AI_SEQ,
+	R_CONS_PAL_AI_ASCII,
+	R_CONS_PAL_AI_UNMAP,
+	R_CONS_PAL_GUI_CFLOW,
+	R_CONS_PAL_GUI_DATAOFFSET,
+	R_CONS_PAL_GUI_BACKGROUND,
+	R_CONS_PAL_GUI_ALT_BACKGROUND,
+	R_CONS_PAL_GUI_BORDER,
+	R_CONS_PAL_GRAPH_BOX,
+	R_CONS_PAL_GRAPH_BOX2,
+	R_CONS_PAL_GRAPH_BOX3,
+	R_CONS_PAL_GRAPH_BOX4,
+	R_CONS_PAL_GRAPH_TRUE,
+	R_CONS_PAL_GRAPH_FALSE,
+	R_CONS_PAL_GRAPH_TRUFAE,
+	R_CONS_PAL_GRAPH_TRACED,
+	R_CONS_PAL_GRAPH_CURRENT,
+	R_CONS_PAL_LAST
+};
+#endif
+
+enum { COLOR_MODE_DISABLED = 0, COLOR_MODE_16, COLOR_MODE_256, COLOR_MODE_16M };
+
+enum { ALPHA_RESET = 0x00, ALPHA_FG = 0x01, ALPHA_BG = 0x02, ALPHA_FGBG = 0x03 };
+enum { R_CONS_ATTR_BOLD = 1 << 1 };
+
+typedef struct rcolor_t {
+	// bold, italic, underline, ...
+	ut8 attr;
+	ut8 a;
+	ut8 r;
+	ut8 g;
+	ut8 b;
+	ut8 r2; // Background color
+	ut8 g2; // Only used when a &= ALPHA_FGBG
+	ut8 b2;
+} RColor;
+
 typedef struct r_cons_palette_t {
+	RColor b0x00;
+	RColor b0x7f;
+	RColor b0xff;
+	RColor args;
+	RColor bin;
+	RColor btext;
+	RColor call;
+	RColor cjmp;
+	RColor cmp;
+	RColor comment;
+	RColor usercomment;
+	RColor creg;
+	RColor flag;
+	RColor fline;
+	RColor floc;
+	RColor flow;
+	RColor flow2;
+	RColor fname;
+	RColor help;
+	RColor input;
+	RColor invalid;
+	RColor jmp;
+	RColor label;
+	RColor math;
+	RColor mov;
+	RColor nop;
+	RColor num;
+	RColor offset;
+	RColor other;
+	RColor pop;
+	RColor prompt;
+	RColor push;
+	RColor crypto;
+	RColor reg;
+	RColor reset;
+	RColor ret;
+	RColor swi;
+	RColor trap;
+	RColor ai_read;
+	RColor ai_write;
+	RColor ai_exec;
+	RColor ai_seq;
+	RColor ai_ascii;
+	RColor gui_cflow;
+	RColor gui_dataoffset;
+	RColor gui_background;
+	RColor gui_alt_background;
+	RColor gui_border;
+	RColor highlight;
+
+	/* Graph colors */
+	RColor graph_box;
+	RColor graph_box2;
+	RColor graph_box3;
+	RColor graph_box4;
+	RColor graph_true;
+	RColor graph_false;
+	RColor graph_trufae;
+	RColor graph_traced;
+	RColor graph_current;
+} RConsPalette;
+
+typedef struct r_cons_printable_palette_t {
 	char *b0x00;
 	char *b0x7f;
 	char *b0xff;
@@ -84,11 +231,13 @@ typedef struct r_cons_palette_t {
 	char *cjmp;
 	char *cmp;
 	char *comment;
+	char *usercomment;
 	char *creg;
 	char *flag;
 	char *fline;
 	char *floc;
 	char *flow;
+	char *flow2;
 	char *fname;
 	char *help;
 	char *input;
@@ -115,11 +264,13 @@ typedef struct r_cons_palette_t {
 	char *ai_exec;
 	char *ai_seq;
 	char *ai_ascii;
+	char *ai_unmap;
 	char *gui_cflow;
 	char *gui_dataoffset;
 	char *gui_background;
 	char *gui_alt_background;
 	char *gui_border;
+	char *highlight;
 
 	/* graph colors */
 	char *graph_box;
@@ -131,10 +282,13 @@ typedef struct r_cons_palette_t {
 	char *graph_trufae;
 	char *graph_traced;
 	char *graph_current;
+	char **rainbow; // rainbow
+	int rainbow_sz; // size of rainbow
+} RConsPrintablePalette;
 
-#define R_CONS_PALETTE_LIST_SIZE 8
-	char *list[R_CONS_PALETTE_LIST_SIZE];
-} RConsPalette;
+R_API char *r_cons_rainbow_get(int idx, int last, bool bg);
+R_API void r_cons_rainbow_free(void);
+R_API void r_cons_rainbow_new(int sz);
 
 typedef void (*RConsEvent)(void *);
 
@@ -172,7 +326,11 @@ typedef struct r_cons_canvas_t {
 #define RUNECODE_CORNER_TL 0xcf
 #define RUNECODE_CORNER_TR 0xd0
 #define RUNECODE_LINE_UP 0xd1
-#define RUNECODE_MAX 0xd2
+#define RUNECODE_CURVE_CORNER_TL 0xd2
+#define RUNECODE_CURVE_CORNER_TR 0xd3
+#define RUNECODE_CURVE_CORNER_BR 0xd4
+#define RUNECODE_CURVE_CORNER_BL 0xd5
+#define RUNECODE_MAX 0xd6
 
 #define RUNECODESTR_MIN 0xc8 // 200
 #define RUNECODESTR_LINE_VERT "\xc8"
@@ -185,7 +343,11 @@ typedef struct r_cons_canvas_t {
 #define RUNECODESTR_CORNER_TL "\xcf"
 #define RUNECODESTR_CORNER_TR "\xd0"
 #define RUNECODESTR_LINE_UP "\xd1"
-#define RUNECODESTR_MAX 0xd1
+#define RUNECODESTR_CURVE_CORNER_TL "\xd2"
+#define RUNECODESTR_CURVE_CORNER_TR "\xd3"
+#define RUNECODESTR_CURVE_CORNER_BR "\xd4"
+#define RUNECODESTR_CURVE_CORNER_BL "\xd5"
+#define RUNECODESTR_MAX 0xd5
 
 #define RUNE_LINE_VERT "│"
 #define RUNE_LINE_CROSS "┼" /* ├ */
@@ -197,16 +359,14 @@ typedef struct r_cons_canvas_t {
 #define RUNE_CORNER_TR "┐"
 #define RUNE_ARROW_RIGHT ">"
 #define RUNE_ARROW_LEFT "<"
+#define RUNE_CURVE_CORNER_TL "╭"
+#define RUNE_CURVE_CORNER_TR "╮"
+#define RUNE_CURVE_CORNER_BR "╯"
+#define RUNE_CURVE_CORNER_BL "╰"
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
 
-#if 0
-r_cons_click_begin();
-r_cons_click_end();
-r_cons_click_clear();
-#endif
-
 typedef struct r_cons_t {
 	RConsGrep grep;
 	RStack *cons_stack;
@@ -228,6 +388,7 @@ typedef struct r_cons_t {
 	int fix_rows;
 	int fix_columns;
 	bool breaked;
+	bool break_lines;
 	int noflush;
 	FILE *fdin; // FILE? and then int ??
 	int fdout; // only used in pipe.c :?? remove?
@@ -244,19 +405,20 @@ typedef struct r_cons_t {
 #if __UNIX__ || __CYGWIN__ && !defined(MINGW32)
 	struct termios term_raw, term_buf;
 #elif __WINDOWS__
-	LPDWORD term_raw, term_buf;
+	DWORD term_raw, term_buf;
 #endif
 	RNum *num;
 	/* Pager (like more or less) to use if the output doesn't fit on the
 	 * current window. If NULL or "" no pager is used. */
 	char *pager;
 	int blankline;
-	int truecolor; // 0 = ansi, 1 = rgb 256), 2 = truecolor (16M)
+	int color; // 0 = none, 1 = ansi (16), 2 = palette (256), 3 = truecolor (16M)
 	char *highlight;
 	int null; // if set, does not show anything
 	int mouse;
 	int is_wine;
-	RConsPalette pal;
+	RConsPalette cpal;
+	RConsPrintablePalette pal;
 	struct r_line_t *line;
 	const char **vline;
 	int refcnt;
@@ -267,11 +429,16 @@ typedef struct r_cons_t {
 #endif
 	bool flush;
 	bool use_utf8; // use utf8 features
+	bool use_utf8_curvy; // use utf8 curved corners
 	int linesleep;
 	int pagesize;
 	char *break_word;
 	int break_word_len;
 	ut64 timeout;
+	bool grep_color;
+	bool use_tts;
+	bool filter;
+	char* (*rgbstr)(char *str, ut64 addr);
 } RCons;
 
 // XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
@@ -305,6 +472,8 @@ typedef struct r_cons_t {
 #define Color_INVERT_RESET "\x1b[27m"
 /* plain colors */
 #define Color_RESET      "\x1b[0m"
+#define Color_NOBGRESET  "\x1b[22;24;25;27;28;39m"
+#define Color_BGRESET    "\x1b[49m"
 #define Color_BLACK      "\x1b[30m"
 #define Color_BGBLACK    "\x1b[40m"
 #define Color_RED        "\x1b[31m"
@@ -335,6 +504,30 @@ typedef struct r_cons_t {
 #define Color_BBLUE     "\x1b[1;34m"
 #define Color_BGRAY     "\x1b[1;38m"
 
+#ifdef _MSC_VER
+#define RCOLOR(a, r, g, b, bgr, bgg, bgb) {0, a, r, g, b, bgr, bgg, bgb}
+#else
+#define RCOLOR(a, r, g, b, bgr, bgg, bgb) (RColor) {0, a, r, g, b, bgr, bgg, bgb}
+#endif
+#define RColor_NULL      RCOLOR(0x00,     0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BLACK     RCOLOR(ALPHA_FG, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BGBLACK   RCOLOR(ALPHA_BG, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_RED       RCOLOR(ALPHA_FG, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BGRED     RCOLOR(ALPHA_BG, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_WHITE     RCOLOR(ALPHA_FG, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BGWHITE   RCOLOR(ALPHA_BG, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00)
+#define RColor_GREEN     RCOLOR(ALPHA_FG, 0x00, 0xff, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BGGREEN   RCOLOR(ALPHA_BG, 0x00, 0xff, 0x00, 0x00, 0x00, 0x00)
+#define RColor_MAGENTA   RCOLOR(ALPHA_FG, 0xff, 0x00, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BGMAGENTA RCOLOR(ALPHA_BG, 0xff, 0x00, 0xff, 0x00, 0x00, 0x00)
+#define RColor_YELLOW    RCOLOR(ALPHA_FG, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BGYELLOW  RCOLOR(ALPHA_BG, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00)
+#define RColor_CYAN      RCOLOR(ALPHA_FG, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BGCYAN    RCOLOR(ALPHA_BG, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BLUE      RCOLOR(ALPHA_FG, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BGBLUE    RCOLOR(ALPHA_BG, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00)
+#define RColor_GRAY      RCOLOR(ALPHA_FG, 0x7f, 0x7f, 0x7f, 0x00, 0x00, 0x00)
+#define RColor_BGGRAY    RCOLOR(ALPHA_BG, 0x7f, 0x7f, 0x7f, 0x00, 0x00, 0x00)
 
 #define Colors_PLAIN { \
 	Color_BLACK, Color_RED, Color_WHITE, \
@@ -370,6 +563,8 @@ enum {
 	LINE_TRUE,
 	LINE_FALSE,
 	LINE_UNCJMP,
+	LINE_NOSYM_VERT,
+	LINE_NOSYM_HORIZ
 };
 
 typedef struct r_cons_canvas_line_style_t {
@@ -405,9 +600,9 @@ typedef struct r_cons_canvas_line_style_t {
 
 
 #ifdef R_API
-R_API RConsCanvas* r_cons_canvas_new (int w, int h);
-R_API void r_cons_canvas_free (RConsCanvas *c);
-R_API void r_cons_canvas_clear (RConsCanvas *c);
+R_API RConsCanvas* r_cons_canvas_new(int w, int h);
+R_API void r_cons_canvas_free(RConsCanvas *c);
+R_API void r_cons_canvas_clear(RConsCanvas *c);
 R_API void r_cons_canvas_print(RConsCanvas *c);
 R_API void r_cons_canvas_print_region(RConsCanvas *c);
 R_API char *r_cons_canvas_to_string(RConsCanvas *c);
@@ -416,20 +611,22 @@ R_API void r_cons_canvas_write(RConsCanvas *c, const char *_s);
 R_API bool r_cons_canvas_gotoxy(RConsCanvas *c, int x, int y);
 R_API void r_cons_canvas_goto_write(RConsCanvas *c,int x,int y, const char * s);
 R_API void r_cons_canvas_box(RConsCanvas *c, int x, int y, int w, int h, const char *color);
-R_API void r_cons_canvas_line (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
-R_API void r_cons_canvas_line_diagonal (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
-R_API void r_cons_canvas_line_square (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
+R_API void r_cons_canvas_line(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
+R_API void r_cons_canvas_line_diagonal(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
+R_API void r_cons_canvas_line_square(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API int r_cons_canvas_resize(RConsCanvas *c, int w, int h);
 R_API void r_cons_canvas_fill(RConsCanvas *c, int x, int y, int w, int h, char ch, int replace);
+R_API void r_cons_canvas_line_square_defined (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int bendpoint, int isvert);
+R_API void r_cons_canvas_line_back_edge (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int ybendpoint1, int xbendpoint, int ybendpoint2, int isvert);
 
-R_API RCons *r_cons_new (void);
-R_API RCons *r_cons_singleton (void);
-R_API RCons *r_cons_free (void);
-R_API char *r_cons_lastline (void);
+R_API RCons *r_cons_new(void);
+R_API RCons *r_cons_singleton(void);
+R_API RCons *r_cons_free(void);
+R_API char *r_cons_lastline(int *size);
 
 typedef void (*RConsBreak)(void *);
 R_API void r_cons_break_end(void);
-R_API bool r_cons_is_breaked();
+R_API bool r_cons_is_breaked(void);
 R_API void r_cons_break_timeout(int timeout);
 R_API void r_cons_breakword(const char *s);
 
@@ -441,19 +638,19 @@ R_API void r_cons_pipe_close(int fd);
 R_API int r_cons_w32_print(const ut8 *ptr, int len, int empty);
 #endif
 
-R_API void r_cons_push();
-R_API void r_cons_pop();
-R_API void r_cons_break_pop();
+R_API void r_cons_push(void);
+R_API void r_cons_pop(void);
+R_API void r_cons_break_pop(void);
 R_API void r_cons_break_push(RConsBreak cb, void*user);
-R_API void r_cons_break_clear();
+R_API void r_cons_break_clear(void);
 
 /* control */
-R_API char *r_cons_editor (const char *file, const char *str);
+R_API char *r_cons_editor(const char *file, const char *str);
 R_API void r_cons_reset(void);
 R_API void r_cons_reset_colors(void);
 R_API void r_cons_print_clear(void);
 R_API void r_cons_zero(void);
-R_API void r_cons_highlight (const char *word);
+R_API void r_cons_highlight(const char *word);
 R_API void r_cons_clear(void);
 R_API void r_cons_clear00(void);
 R_API void r_cons_clear_line(int err);
@@ -461,21 +658,22 @@ R_API void r_cons_fill_line(void);
 R_API void r_cons_stdout_open(const char *file, int append);
 R_API int  r_cons_stdout_set_fd(int fd);
 R_API void r_cons_gotoxy(int x, int y);
-R_API void r_cons_show_cursor (int cursor);
+R_API void r_cons_show_cursor(int cursor);
 R_API char *r_cons_swap_ground(const char *col);
-R_API bool r_cons_drop (int n);
-R_API void r_cons_chop ();
-R_API void r_cons_set_raw(int b);
-R_API void r_cons_set_interactive(int b);
+R_API bool r_cons_drop(int n);
+R_API void r_cons_chop(void);
+R_API void r_cons_set_raw(bool b);
+R_API void r_cons_set_interactive(bool b);
 R_API void r_cons_set_last_interactive(void);
 
 /* output */
 R_API void r_cons_printf(const char *format, ...);
+R_API void r_cons_printf_list(const char *format, va_list ap);
 R_API void r_cons_strcat(const char *str);
 #define r_cons_print(x) r_cons_strcat (x)
 R_API void r_cons_println(const char* str);
-R_API void r_cons_strcat_justify (const char *str, int j, char c);
-R_API void r_cons_memcat(const char *str, int len);
+R_API void r_cons_strcat_justify(const char *str, int j, char c);
+R_API int r_cons_memcat(const char *str, int len);
 R_API void r_cons_newline(void);
 R_API void r_cons_filter(void);
 R_API void r_cons_flush(void);
@@ -484,60 +682,69 @@ R_API void r_cons_less(void);
 R_API void r_cons_2048(bool color);
 R_API void r_cons_memset(char ch, int len);
 R_API void r_cons_visual_flush(void);
-R_API void r_cons_visual_write (char *buffer);
+R_API void r_cons_visual_write(char *buffer);
 R_API int r_cons_is_utf8(void);
+R_API void r_cons_cmd_help(const char * help[], bool use_color);
 
 /* input */
 //R_API int  r_cons_fgets(char *buf, int len, int argc, const char **argv);
 R_API int r_cons_controlz(int ch);
 R_API int r_cons_readchar(void);
+R_API bool r_cons_readpush(const char *str, int len);
+R_API void r_cons_readflush();
+R_API void r_cons_switchbuf(bool active);
+R_API int r_cons_readchar_timeout(ut32 usec);
 R_API int r_cons_any_key(const char *msg);
 R_API int r_cons_eof(void);
 
 R_API int r_cons_palette_init(const unsigned char *pal);
-R_API int r_cons_pal_set (const char *key, const char *val);
+R_API int r_cons_pal_set(const char *key, const char *val);
+R_API void r_cons_pal_update_event(void);
 R_API void r_cons_pal_free(void);
-R_API void r_cons_pal_init(const char *foo);
-R_API char *r_cons_pal_parse(const char *str);
+R_API void r_cons_pal_init();
+R_API char *r_cons_pal_parse(const char *str, RColor *outcol);
 R_API void r_cons_pal_random(void);
-R_API const char *r_cons_pal_get (const char *key);
-R_API const char *r_cons_pal_get_i (int n);
-R_API const char *r_cons_pal_get_color(int n);
-R_API int r_cons_rgb_parse (const char *p, ut8 *r, ut8 *g, ut8 *b, int *is_bg);
+R_API RColor r_cons_pal_get(const char *key);
+R_API RColor r_cons_pal_get_i(int index);
+R_API const char *r_cons_pal_get_name(int index);
+R_API int r_cons_rgb_parse(const char *p, ut8 *r, ut8 *g, ut8 *b, ut8 *a);
 R_API char *r_cons_rgb_tostring(ut8 r, ut8 g, ut8 b);
-R_API void r_cons_pal_list (int rad, const char *arg);
-R_API void r_cons_pal_show (void);
+R_API void r_cons_pal_list(int rad, const char *arg);
+R_API void r_cons_pal_show(void);
 R_API int r_cons_get_size(int *rows);
-R_API bool r_cons_isatty();
+R_API bool r_cons_isatty(void);
 R_API int r_cons_get_cursor(int *rows);
 R_API int r_cons_arrow_to_hjkl(int ch);
-R_API int r_cons_html_print(const char *ptr);
+R_API char *r_cons_html_filter(const char *ptr, int *newlen);
 
 // TODO: use gets() .. MUST BE DEPRECATED
 R_API int r_cons_fgets(char *buf, int len, int argc, const char **argv);
-R_API char *r_cons_hud(RList *list, const char *prompt, const bool usecolor);
-R_API char *r_cons_hud_path(const char *path, int dir, const bool usecolor);
-R_API char *r_cons_hud_string(const char *s, const bool usecolor);
-R_API char *r_cons_hud_file(const char *f, const bool usecolor);
+R_API char *r_cons_hud(RList *list, const char *prompt);
+R_API char *r_cons_hud_path(const char *path, int dir);
+R_API char *r_cons_hud_string(const char *s);
+R_API char *r_cons_hud_file(const char *f);
 
 R_API const char *r_cons_get_buffer(void);
-R_API void r_cons_grep(const char *str);
+R_API void r_cons_grep_help(void);
+R_API void r_cons_grep_parsecmd(char *cmd, const char *quotestr);
+R_API char * r_cons_grep_strip(char *cmd, const char *quotestr);
+R_API void r_cons_grep_process(char * grep);
 R_API int r_cons_grep_line(char *buf, int len); // must be static
 R_API int r_cons_grepbuf(char *buf, int len);
 
-R_API void r_cons_rgb (ut8 r, ut8 g, ut8 b, int is_bg);
-R_API void r_cons_rgb_fgbg (ut8 r, ut8 g, ut8 b, ut8 R, ut8 G, ut8 B);
-R_API void r_cons_rgb_init (void);
-R_API char *r_cons_rgb_str (char *outstr, ut8 r, ut8 g, ut8 b, int is_bg);
-R_API void r_cons_color (int fg, int r, int g, int b);
-R_API char *r_cons_color_random(int bg);
-R_API char *r_cons_color_random_string(int bg);
+R_API void r_cons_rgb(ut8 r, ut8 g, ut8 b, ut8 a);
+R_API void r_cons_rgb_fgbg(ut8 r, ut8 g, ut8 b, ut8 R, ut8 G, ut8 B);
+R_API void r_cons_rgb_init(void);
+R_API char *r_cons_rgb_str(char *outstr, RColor *rcolor);
+R_API char *r_cons_rgb_str_off(char *outstr, ut64 off);
+R_API void r_cons_color(int fg, int r, int g, int b);
+R_API RColor r_cons_color_random(ut8 alpha);
 R_API void r_cons_invert(int set, int color);
 R_API int r_cons_yesno(int def, const char *fmt, ...);
 R_API char *r_cons_input(const char *msg);
 R_API void r_cons_set_cup(int enable);
 R_API void r_cons_column(int c);
-R_API int r_cons_get_column (void);
+R_API int r_cons_get_column(void);
 R_API char *r_cons_message(const char *msg);
 R_API void r_cons_set_title(const char *str);
 R_API bool r_cons_enable_mouse(const bool enable);
@@ -575,9 +782,14 @@ typedef struct r_line_comp_t {
 
 typedef char* (*RLineEditorCb)(void *core, const char *str);
 
+typedef int (*RLineHistoryUpCb)(RLine* line);
+typedef int (*RLineHistoryDownCb)(RLine* line);
+
 struct r_line_t {
 	RLineCompletion completion;
 	RLineHistory history;
+	RLineHistoryUpCb history_up_cb;
+	RLineHistoryDownCb history_down_cb;
 	RLineBuffer buffer;
 	RLineEditorCb editor_cb;
 	int echo;
@@ -590,6 +802,8 @@ struct r_line_t {
 	int (*hist_down)(void *user);
 	char *contents;
 	bool zerosep;
+	bool offset_prompt;
+	int offset_index;
 }; /* RLine */
 
 #ifdef R_API
@@ -597,12 +811,12 @@ struct r_line_t {
 R_API RLine *r_line_new(void);
 R_API RLine *r_line_singleton(void);
 R_API void r_line_free(void);
-R_API char *r_line_get_prompt (void);
+R_API char *r_line_get_prompt(void);
 R_API void r_line_set_prompt(const char *prompt);
-R_API int r_line_dietline_init();
-R_API void r_line_hist_free();
+R_API int r_line_dietline_init(void);
+R_API void r_line_hist_free(void);
 
-typedef int (RLineReadCallback) (void *user, const char *line);
+typedef int (RLineReadCallback)(void *user, const char *line);
 R_API const char *r_line_readline(void);
 R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user);
 
@@ -614,6 +828,10 @@ R_API void r_line_label_show(void);
 R_API int r_line_hist_list(void);
 R_API const char *r_line_hist_get(int n);
 
+R_API int r_line_set_hist_callback(RLine *line, RLineHistoryUpCb cb_up, RLineHistoryDownCb cb_down);
+R_API int cmd_history_up(RLine *line);
+R_API int cmd_history_down(RLine *line);
+
 #define R_CONS_INVERT(x,y) (y? (x?Color_INVERT: Color_INVERT_RESET): (x?"[":"]"))
 
 #endif
@@ -631,18 +849,21 @@ typedef struct r_ascii_node_t {
 	int h;
 
 	int layer;
+	int layer_height;
+	int layer_width;
 	int pos_in_layer;
 	int is_dummy;
 	int is_reversed;
 	int klass;
-	bool mini;
+	bool is_mini;
 } RANode;
 
 #define R_AGRAPH_MODE_NORMAL 0
 #define R_AGRAPH_MODE_OFFSET 1
 #define R_AGRAPH_MODE_MINI 2
-#define R_AGRAPH_MODE_SUMMARY 3
-#define R_AGRAPH_MODE_MAX 4
+#define R_AGRAPH_MODE_TINY 3
+#define R_AGRAPH_MODE_SUMMARY 4
+#define R_AGRAPH_MODE_MAX 5
 
 typedef void (*RANodeCallback)(RANode *n, void *user);
 typedef void (*RAEdgeCallback)(RANode *from, RANode *to, void *user);
@@ -655,9 +876,13 @@ typedef struct r_ascii_graph_t {
 	Sdb *db;
 	Sdb *nodes; // Sdb with title(key)=RANode*(value)
 
+	int layout;
 	int is_instep;
+	bool is_tiny;
+	bool is_dis;
+	int edgemode;
 	int mode;
-	int is_callgraph;
+	bool is_callgraph;
 	int zoom;
 	int movspeed;
 
@@ -681,13 +906,6 @@ typedef struct r_ascii_graph_t {
 	int n_layers;
 	RList *dists; /* RList<struct dist_t> */
 	RList *edges; /* RList<AEdge> */
-
-	/* colors */
-	const char *color_box;
-	const char *color_box2;
-	const char *color_box3;
-	const char *color_true;
-	const char *color_false;
 } RAGraph;
 
 #ifdef R_API
@@ -709,6 +927,24 @@ R_API void r_agraph_foreach_edge(RAGraph *g, RAEdgeCallback cb, void *user);
 R_API void r_agraph_set_curnode(RAGraph *g, RANode *node);
 #endif
 
+typedef struct r_panels_t {
+	RConsCanvas *can;
+	RPanel *panel;
+	int n_panels;
+	int columnWidth;
+	int layout;
+	int menu_pos;
+	int menu_x;
+	int menu_y;
+	int callgraph;
+	int curnode;
+	bool isResizing;
+} RPanels;
+#ifdef R_API
+R_API RPanels *r_panels_new();
+R_API void r_panels_free(RPanels *panels);
+#endif
+
 #ifdef __cplusplus
 }
 #endif
