@@ -26,6 +26,8 @@
 int gettimeofday (struct timeval* p, void* tz);
 #endif
 #include <sys/time.h>
+#include "r_util/r_itv.h"
+#include "r_util/r_rbtree.h"
 #include "r_util/r_big.h"
 #include "r_util/r_base64.h"
 #include "r_util/r_base91.h"
@@ -43,6 +45,7 @@ int gettimeofday (struct timeval* p, void* tz);
 #include "r_util/r_name.h"
 #include "r_util/r_num.h"
 #include "r_util/r_graph.h"
+#include "r_util/r_panels.h"
 #include "r_util/r_pool.h"
 #include "r_util/r_punycode.h"
 #include "r_util/r_queue.h"
@@ -52,12 +55,15 @@ int gettimeofday (struct timeval* p, void* tz);
 #include "r_util/r_spaces.h"
 #include "r_util/r_stack.h"
 #include "r_util/r_str.h"
+#include "r_util/r_ascii_table.h"
 #include "r_util/r_strbuf.h"
 #include "r_util/r_strpool.h"
 #include "r_util/r_sys.h"
 #include "r_util/r_tree.h"
 #include "r_util/r_uleb128.h"
 #include "r_util/r_utf8.h"
+#include "r_util/r_utf16.h"
+#include "r_util/r_utf32.h"
 #include "r_util/r_id_storage.h"
 #include "r_util/r_asn1.h"
 #include "r_util/r_json.h"
