@@ -1,10 +1,17 @@
-/* radare2 - LGPL - Copyright 2017 - condret */
+/* radare2 - LGPL - Copyright 2017-2018 - condret */
 
 #ifndef R_ID_STORAGE_H
 #define R_ID_STORAGE_H
+
+#include <r_util/r_pool.h>
+#include <r_util/r_queue.h>
 #include <r_util.h>
 #include <r_types.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct r_id_pool_t {
 	ut32 start_id;
 	ut32 last_id;
@@ -34,4 +41,10 @@ R_API void r_id_storage_delete(RIDStorage *storage, ut32 id);
 R_API void *r_id_storage_take(RIDStorage *storage, ut32 id);
 R_API bool r_id_storage_foreach(RIDStorage *storage, RIDStorageForeachCb cb, void *user);
 R_API void r_id_storage_free(RIDStorage *storage);
+R_API RList *r_id_storage_list(RIDStorage *s);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
